/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.test;

import junit.framework.TestCase;

import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.util.Date;
import java.util.Properties;


/**
 * Test case for queue rollback
 *
 * @version $Revision: 1.1 $
 * @author	Deepak.Nayal
 */
public class RollbackTest extends TestCase implements MessageListener {


    public final String JMS_FACTORY = "QueueConnectionFactory";
    public final String QUEUE = "DNQueue";

    Context ctx;
    QueueConnectionFactory qfact;
    QueueConnection qcon;
    QueueSession sess;
    Queue q;
    QueueReceiver qrec;

    private int commitCount;
    private int rollbackCount;
    private JMSException exception;
    private Object lock = new Object();


    public void testRun() throws Exception {
        sendMessages();

        // lets start the connection after sending the messages
        // as it makes it easier to see whats going on in a console
        qcon.start();

        for (int i = 0; i < 10; i++) {
            synchronized (lock) {
                lock.wait(2000);
            }

            if (rollbackCount >= 4) {
                break;
            }
        }

        assertTrue("Should have rolled back at least twice, but was: " + rollbackCount, rollbackCount >= 4);
        assertEquals("Should have commit twice", 2, commitCount);
        assertEquals("Should not have caught an exception", null, exception);

        System.out.println("DONE");
    }

    public void onMessage(Message msg) {
        TextMessage tm = (TextMessage) msg;
        try {
            String text = tm.getText();
            System.out.println("Received MSG : " + text);

            if (text.startsWith("5") || text.startsWith("10")) {
                System.out.println("Committing!");
                sess.commit();
                commitCount++;
            }
            else if (text.startsWith("9")) {
                System.out.println("Rolling back!");
                sess.rollback();
                rollbackCount++;

                synchronized (lock) {
                    lock.notify();
                }
            }
        }
        catch (JMSException e) {
            System.out.println("Exception: " + e);
            e.printStackTrace();
            exception = e;
        }
    }


    protected void setUp() throws Exception {
        Properties env = new Properties();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "org.activemq.jndi.ActiveMQInitialContextFactory");
        env.put(Context.PROVIDER_URL, "vm://localhost");
        env.put("queue.DNQueue", getClass().getName());

//        env.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
//        env.put(Context.PROVIDER_URL, "t3://localhost:7001");
        ctx = new InitialContext(env);
        qfact = (QueueConnectionFactory) ctx.lookup(JMS_FACTORY);
        qcon = qfact.createQueueConnection();
//        sess = qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
        sess = qcon.createQueueSession(true, -1);
        q = (Queue) ctx.lookup(QUEUE);
        assertNotNull(q);

        qrec = sess.createReceiver(q);
        qrec.setMessageListener(this);
    }

    protected void tearDown() throws Exception {
        sess.close();
        qcon.close();
    }


    public void sendMessages() throws Exception {
        QueueSession sess = qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
        QueueSender qsend = sess.createSender(q);

        for (int i = 1; i <= 10; i++) {
            String data = "" + i + " " + new Date();
            TextMessage tm = sess.createTextMessage();
            tm.setText(data);
            qsend.send(tm);
            System.out.println("Message Sent: " + data);
        }
        qsend.close();
        sess.close();
    }


}
